OpenLayers.Util.extend(OpenLayers.Lang.fr, {
    "layertree": "Thèmes",
    "blank": "Fond blanc",

    /* Print */
    "title": "Titre",
    "comment": "Comentaires",
    "debug": "Ajouter en-tête de debug"
});
